package com.sundata.internalevaluation.configuration.service;

import com.sundata.internalevaluation.configuration.mapper.ModelConfigMapper;
import com.sundata.internalevaluation.configuration.model.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

@Service
public class ModelConfigService {

    @Autowired
    ModelConfigMapper modelConfigMapper;

    /**
     * @author: 张艳杰
     * @methodsName: getDataList
     * @description: 数据来源配置初始化
     * @param: ModelConfigModel model
     * @return: List<ModelConfigModel>
     * @throws:
     */
    public List<ModelConfigModel> getDataList(ModelConfigModel model) {
        return modelConfigMapper.getDataList(model);
    }

    /**
     * @author: 张艳杰
     * @methodsName: createData
     * @description: 模型配置新增
     * @param: ModelConfigModel model
     * @return:
     * @throws:
     */
    public void createData(ModelConfigModel model) {
        modelConfigMapper.createData(model);
        for(IndexListModel m : model.getIndexList()){
            m.setModelNo(model.getModelNo());
        }
        modelConfigMapper.createIndexListData(model.getIndexList());
        for(ModelGradeModel m : model.getModelGradeList()){
            m.setModelNo(model.getModelNo());
        }
        modelConfigMapper.createModelGradeData(model.getModelGradeList());
    }

    /**x
     * @author: 张艳杰
     * @methodsName: deleteRows
     * @description: 模型配置删除
     * @param: ModelConfigModel model
     * @return:
     * @throws:
     */
    public void deleteRows(ModelConfigModel model) {
        modelConfigMapper.deleteRows(model);
    }

    /**
     * @author: 张艳杰
     * @methodsName: updateExistData
     * @description: 模型配置修改
     * @param: ModelConfigModel model
     * @return:
     * @throws:
     */
    public void updateExistData(ModelConfigModel model) {
        modelConfigMapper.updateExistData(model);
        modelConfigMapper.deleteIndexListRows(model);
        modelConfigMapper.deleteModelGradeRows(model);
        for(IndexListModel m : model.getIndexList()){
            m.setModelNo(model.getModelNo());
        }
        modelConfigMapper.createIndexListData(model.getIndexList());
        for(ModelGradeModel m : model.getModelGradeList()){
            m.setModelNo(model.getModelNo());
        }
        modelConfigMapper.createModelGradeData(model.getModelGradeList());
    }

    /**
     * @author: 张艳杰
     * @methodsName: selectDetailData
     * @description: 模型配置查询
     * @param:  ModelConfigModel model
     * @return:
     * @throws:
     */
    public ModelConfigModel selectDetailData(ModelConfigModel model) {
        List<IndexListModel> indexList = modelConfigMapper.getIndexDataList(model);
        List<ModelGradeModel> modelGradeList = modelConfigMapper.getModelGradeList(model);
        model.setIndexList(indexList);
        model.setModelGradeList(modelGradeList);
        return model;
    }
}
