package com.sundata.internalevaluation.configuration.service;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.sundata.common.exception.BusinessException;
import com.sundata.internalevaluation.configuration.mapper.SysReqRulesMapper;
import com.sundata.internalevaluation.configuration.model.SysReqRuleRules;
import com.sundata.internalevaluation.configuration.model.SysReqRules;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DuplicateKeyException;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.stream.Collectors;

/**
 * @author JoeLazy
 */
@Service
public class SysReqRulesService {

    @Autowired
    private SysReqRulesMapper sysReqRulesMapper;

    /**
     * 查询所有记录
     *
     * @return 返回集合，没有返回空List
     */
    public List<SysReqRules> listAll(SysReqRules sysReqRules) {
        return sysReqRulesMapper.listAll(sysReqRules);
    }


    /**
     * 根据主键查询
     *
     * @param rulesNo 主键
     * @return 返回记录，没有返回null
     */

    static final Logger log = LoggerFactory.getLogger(SysReqRulesService.class);

    public SysReqRules getById(String rulesNo) {

        SysReqRules rules = sysReqRulesMapper.getById(rulesNo);
        List<String> rulesIds = sysReqRulesMapper.selectRuleByRulesNo(rulesNo);
        rules.setRuleIds(rulesIds);
        if (rulesIds == null || rulesIds.isEmpty()) {
            log.error("暂时没有规则信息");
//            throw new BusinessException("失败了，没有规则信息");
        }
        return rules;
    }


    /**
     * 新增，忽略null字段
     *
     * @param sysReqRules 新增的记录
     * @return 返回影响行数
     */
    @Transactional
    public int insertIgnoreNull(SysReqRules sysReqRules) throws JsonProcessingException {

        try {
            // 规则集编号
            String rulesNo = sysReqRules.getRulesNo();
            // 删除绑定关系
            sysReqRulesMapper.deleteAssociateRuleByRulesNo(rulesNo);
            // 规则列表
            List<String> ruleIds = sysReqRules.getRuleIds();
            if (ruleIds != null && !ruleIds.isEmpty()) {
                List<SysReqRuleRules> associateRules = ruleIds.stream().map(ruleId -> new SysReqRuleRules(ruleId, rulesNo)).collect(Collectors.toList());
                sysReqRulesMapper.insertAssociateRule(associateRules);
            }

            return sysReqRulesMapper.insertIgnoreNull(sysReqRules);
        } catch (DuplicateKeyException e) {
            throw new BusinessException("规则编码已存在，请重新输入.");
        }
    }


    /**
     * 修改，忽略null字段
     *
     * @param sysReqRules 修改的记录
     * @return 返回影响行数
     */
    @Transactional
    public int updateIgnoreNull(SysReqRules sysReqRules) {

        // 规则集编号
        String rulesNo = sysReqRules.getRulesNo();
        // 删除绑定关系
        sysReqRulesMapper.deleteAssociateRuleByRulesNo(rulesNo);


        // 规则列表
        List<String> ruleIds = sysReqRules.getRuleIds();
        if (ruleIds != null && !ruleIds.isEmpty()) {
            List<SysReqRuleRules> associateRules = ruleIds.stream().map(ruleId -> new SysReqRuleRules(ruleId, rulesNo)).collect(Collectors.toList());
            sysReqRulesMapper.insertAssociateRule(associateRules);
        }

        return sysReqRulesMapper.updateIgnoreNull(sysReqRules);
    }

    /**
     * 删除记录
     *
     * @param sysReqRules 待删除的记录
     * @return 返回影响行数
     */

    @Transactional
    public int delete(SysReqRules sysReqRules) {
        sysReqRulesMapper.deleteAssociateRuleByRulesNo(sysReqRules.getRulesNo());
        return sysReqRulesMapper.delete(sysReqRules);
    }

}